% computes averages of windowed power spectrum of the images in the set
% Reference: Torralba, A., Oliva, A. (2003). Statistics of natural images
% categories. Network: computation in neural systems, 14, 391-412.

clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Put here the name of the folder that contains the set of images:

folder = 'plaids/';
% Number of windows:
Nwindows = 4; % Use powers of 2 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

files = [dir([folder '*.jpg']); dir([folder '*.tif']); dir([folder '*.tiff']); dir([folder '*.bmp'])]; 

Nimages = length(files);

% Window to remove boundary artifacts (for each subwindow)
g = hammingfn(256/Nwindows)*hammingfn(256/Nwindows)'; % Here I assume that images are 256x256


averageFFT = 0;
figure
x = (linspace(0, 256, Nwindows+1)); % Non-overlaping windows

% Loop: read images and compute averages
for i = 1:Nimages
    % real image
    img = imread([folder files(i).name]);
    %img = imresize(img, [256 256]); % if your images have different sizes
    % you will have to normalize them here... I assume that your images are
    % square and 256x256
    subplot(121)
    imagesc(img); axis('equal'); axis('tight'); drawnow
    
    % Compute FFT and power spectrum
    img = double(img);
    imgBW = mean(img, 3); % put image in gray levels
    
    fftImg = zeros(size(imgBW));
    for n = 1:Nwindows % scan X
        for m = 1:Nwindows % scan Y
            crop = imgBW(x(m)+1:x(m+1), x(n)+1:x(n+1));
            crop = crop - mean(crop(:)); % remove local mean
            crop = crop / std(crop(:)+1); % we can normalize also the local contrast
            fftImg(x(m)+1:x(m+1), x(n)+1:x(n+1))=fftshift(abs(fft2(crop.*g)).^2); % compute power spectrum
        end
    end
    
    subplot(122)
    imagesc(log(fftImg)); axis('equal'); axis('tight'); colormap(gray(256)); drawnow
    
    averageFFT=averageFFT+fftImg;    % power spectrum average
end
averageFFT = averageFFT/Nimages;

figure
surf(log(averageFFT)); shading interp; colormap(gray(256)); axis('tight'); axis('ij')
xlabel('x, f_x'); ylabel('y, f_y'); zlabel('log'); view(-15,70)
title('power spectrum average')

% % Plot sections:
figure
for n = 1:Nwindows % scan X
    for m = 1:Nwindows % scan Y
        crop = averageFFT(x(m)+1:x(m+1), x(n)+1:x(n+1));
        crop = crop/sum(crop(:));
        [thresholds, ndx] = sort(-crop(:)); thresholds= -thresholds;
        energySections = cumsum(thresholds);
        [foo, ndx90] = min(abs(energySections - .9)); th90 = thresholds(ndx90);
        [foo, ndx80] = min(abs(energySections - .8)); th80 = thresholds(ndx80);
        [foo, ndx60] = min(abs(energySections - .6)); th60 = thresholds(ndx60);
        
        if ~isnan(th90)
            subplot(Nwindows, Nwindows, n+(m-1)*Nwindows)
            [c,h] = contour(crop,[th90 th80 th60]); % Here I assume that images are 256x256
            axis('square'); axis('off')
        end
    end
end
